<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');


class Tasks_member extends CI_Model {

	private $entity_id	= null;
	private $user_id	= null;

	private $closed_status_id	= 0;
	private $open_status_id	= 0;

	function __construct()
	{
        parent::__construct();

        $this->entity_id	= $this->ot_app->get('entity_id');
        $this->user_id	= $this->ot_user->get('user_id');
	}

	public function ts_hrs($data)
	{
		$result = array();
		switch ($data['type'])
		{
			default:
				$select = 'ts.user_id, ts.project_task_id, sum(ts.hours) AS hours';
				$where	= array(
					'ts.project_task_id >'	=> 0,
					'ts.entity_id'			=> $this->entity_id,
					'ts.project_id'			=> $data['project_id']
				);

				if (!empty($data['start_date']))
				{
					$where['ts.entry_date >= ']	= $data['start_date'];
				}

				if (!empty($data['end_date']))
				{
					$where['ts.entry_date <= ']	= $data['end_date'];
				}

				$this->db->select($select);
				$this->db->from('time_sheet as ts');
				$this->db->where($where);
				$groupBy	= 'ts.project_task_id, ts.user_id';
				if ($data['type'] == 'ACTUAL')
				{
					if (!empty($data['project_task_id']))
					{
						$this->db->where('project_task_id', $data['project_task_id']);
						$this->db->order_by('hours', 'desc');
					}
					else
					{
						$groupBy	= 'ts.project_task_id';
					}
				}

				$this->db->group_by($groupBy);

				$query = $this->db->get();
				$timesheet = $query->result_array();

				if ($timesheet)
				{
					$result['items']	= $this->response->prepare($timesheet, true);

					if (($data['type'] != 'ACTUAL' || empty($data['filter_by'])) && empty($data['project_task_id']))
					{
						// get users details
						$this->db->select('user_id, first_name, last_name, user_image');
						$this->db->from('entity_users');
						$this->db->where('entity_id', $this->entity_id);

						$query = $this->db->get();
						$result['users'] = $this->response->prepare($query->result_array(), true);
					}
				}
			break;
		}
		
		return $result;
	}

	public function get_task_number ($project_id)
	{
		$this->db->select('MAX(task_number)+1 AS next_task_number');
		$this->db->from('project_task');
		$this->db->where(array('entity_id' => $this->entity_id , 'project_id' => $project_id));
		
		$query = $this->db->get();
		$row = $query->row();
		
		$next_task_number = 1;
		
		if ($row->next_task_number)
			$next_task_number = $row->next_task_number;
		
		return $next_task_number;
	}

	// return tasks in as array of standard array indexes
	public function get_tasks($select = '', $options = array())
	{
		$select= ($select)?$select:'ts.task_id as id, ts.*';
		
		$query = $this->tasks_list_query($select, $options);
		$items	= $query->result_array();
		
		return $items;
	}

	// return tasks list query
	public function tasks_list_query($select, $options = array())
	{
		$this->db->select($select);
		$this->db->select("DATE_FORMAT(ts.start_date, '%Y-%m-%d') as start_date", false);
		$this->db->select("DATE_FORMAT(ts.finish_date, '%Y-%m-%d') as finish_date", false);
		$this->db->from('project_task as ts');
		$where		= array();
		$where	= array('ts.entity_id' => $this->entity_id);
		if (!empty($options['filters']))
		{
			$where	= array_merge($where, $options['filters']);
		}

		if ($where)
		{
			foreach ($where as $key=>$val)
			{
				if (is_array($val))
				{
					$this->db->where_in($key, $val);
				}
				else
				{
					$this->db->where($key, $val);
				}
			}
		}

		$order_by 			= !empty($options['order_by']) ? $options['order_by'] : 'ts.task_id';
		$order_by_direction = !empty($options['order_by_direction']) ? $options['order_by_direction'] : 'asc';

		$this->db->order_by($order_by, $order_by_direction);
		if (!empty($options['group_by']))
				$this->db->group_by($options['group_by']);
		$from	= !empty($options['from']) ? $options['from'] : 0;
		if (!empty($options['count']))
		{
			$this->db->limit($options['count'], $from);
		}

		return $this->db->get(); 
	}
/*
	public function get_project_members($project_id)
	{
		$this->db->select('pru.user_id, u.first_name, u.last_name, u.is_active');
		$this->db->from('project as pr');
		$this->db->join('project_users as pru', 'pr.entity_id=pru.entity_id and pr.project_id=pru.project_id', 'INNER');
		$this->db->join('entity_users as u', 'pru.user_id=u.user_id', 'INNER');
		$this->db->where(array('pr.entity_id' => $this->entity_id, 'pr.project_id' => $project_id));
		$this->db->where('pru.user_id', $this->user_id);

		$this->db->order_by('u.first_name, u.last_name');

		$query = $this->db->get();
		return $query->result_array();
	}
*/
	public function get_project_members($project_id, $select='')
	{
		$this->db->select($select ? $select : 'pru.user_id, pru.user_role, u.first_name, u.last_name, u.is_active, u.user_image');
		$this->db->from('project as pr');
		$this->db->join('project_users as pru', 'pr.entity_id=pru.entity_id and pr.project_id=pru.project_id', 'INNER');
		$this->db->join('entity_users as u', 'pru.user_id=u.user_id', 'INNER');
		$this->db->where(array('pr.entity_id' => $this->entity_id, 'pr.project_id' => $project_id));
		$this->db->order_by('u.first_name, u.last_name');
	
		$query = $this->db->get();
		return $query->result_array();
	}

	public function get_project_tasktypes($project_id, $select='')
	{
		$this->db->select($select ? $select : 'ptt.task_type_id');
		$this->db->from('project_task_types as ptt');
		$this->db->where(array('ptt.entity_id' => $this->entity_id, 'ptt.project_id' => $project_id));

		$query = $this->db->get();
		return $query->result_array();
	}

	public function get_all_users()
	{
		$this->db->select('user_id, first_name, last_name, is_active, user_image');
		$this->db->from('entity_users');
		$this->db->where('entity_id', $this->entity_id);
		$this->db->order_by('first_name, last_name');
// 		$this->db->where('is_active', '1');

		$query	= $this->db->get();
		$return	= $query->result_array();

		return $return;
	}

	public function get_followers($project_id, $where = array())
	{
		$this->db->select('ptu.task_id, GROUP_CONCAT( ptu.user_id SEPARATOR "-") AS users', false);
		$this->db->from('project_task_users AS ptu');
		$this->db->join('project_task AS pt', 'ptu.entity_id=pt.entity_id and ptu.task_id = pt.task_id');
		$this->db->where('pt.project_id',$project_id);

		if ($where)
		{
			foreach ($where as $key=>$val)
			{
				if (is_array($val))
				{
					$this->db->where_in($key, $val);
				}
				else
				{
					$this->db->where($key, $val);
				}
			}
		}

		$this->db->group_by('ptu.task_id');

		$query = $this->db->get();
		return $query->result_array();
	}

	public function get_new_count($project_id)
	{
		$this->db->select('ptu.task_id, ptu.new_comments', false);
		$this->db->from('project_task_users AS ptu');
		$this->db->join('project_task AS pt', 'ptu.entity_id=pt.entity_id and ptu.task_id = pt.task_id');
		$this->db->where(array('pt.entity_id' => $this->entity_id, 'pt.project_id' => $project_id, 'ptu.user_id' => $this->user_id));

		$query = $this->db->get();
		$result = $query->result_array();

		return $result;
	}

	public function save_document($task_id, $data)
	{
		$this->load->library('attachment_table');

		if(!empty($data['new_attachment'])) 
		{
			foreach($data['new_attachment'] as $key=>$value) 
			{
				$attachment_id	= $this->attachment_table->save($value);
				if($value['attachment_id']==0 && $attachment_id) 
				{
					$this->update_has_documents($task_id, '1');
					return $attachment_id;
				}
			}
		}
		return true;
	}

	public function update_has_documents($task_id, $value='1')
	{
		$this->db->where(array(
			'entity_id' => $this->entity_id,
			'task_id'	=> $task_id
		));

		$this->db->update('project_task', array('has_attachments' => $value));
	}

	public function update_worked_hours($task_id=0)
	{
		if (!$task_id)
		{
			return false;
		}

		$this->db->select_sum('hours');
		$this->db->where(array(
			'entity_id' => $this->entity_id,
			'project_task_id'	=> $task_id
		));
		$query	= $this->db->get('time_sheet');
		$result	= $query->result_array();

		if (!empty($result) && !empty($result[0]['hours']))
		{
			$worked_hours	= $result[0]['hours'];

			$this->db->where(array(
				'entity_id' => $this->entity_id,
				'task_id'	=> $task_id
			));

			$this->db->update('project_task', array('worked_hours' => $worked_hours));
		}
	}

	public function save($data)
	{
		$this->load->library('tasks_table');

		$result	= $this->tasks_table->save($data); 
		if (!$result)
		{
			return false;
		}

		return $result;
	}

	public function remove_notify($project_id)
	{
		$this->db->where('entity_id', $this->entity_id);
		$this->db->where('user_id', $this->user_id);
		$this->db->where('project_id', $project_id);
		// Delete all task related notification records.
		$this->db->where_in('type', array('TA', 'TD', 'TP', 'TS', 'TU'));
		$this->db->delete('notification');
	}

	private function get_status_id($is_closed=false)
	{
		if ($is_closed && $this->closed_status_id)
		{
			return $this->closed_status_id;
		}

		if (!$is_closed && $this->open_status_id)
		{
			return $this->open_status_id;
		}

		$result	= $this->db->select('task_status_id, is_closed')->from('task_status')->where(array(
			'entity_id'	=> $this->entity_id,
			'system_defined'	=> '1',
// 			'is_closed'		=> $is_closed ? '1' : '0'
		))->get()->result_array();
		if (!empty($result))
		{
			foreach ($result as $key=>$val)
			{
				if ($val['is_closed'])
				{
					$this->closed_status_id	= $val['task_status_id'];
				}
				else
				{
					$this->open_status_id	= $val['task_status_id'];
				}
			}
		}

		if ($is_closed)
		{
			return $this->closed_status_id;
		}
		else
		{
			return $this->open_status_id;
		}
	}

	public function get_followers_by_task($task_ids=array())
	{
		$this->db->select('task_id, user_id')->from('project_task_users');
		$this->db->where('entity_id', $this->entity_id);
		$this->db->where_in('task_id', $task_ids);
		
		return $this->db->get()->result_array();
	}

	public function get_over_due($count_only=true)
	{
		$closed_status_id	= $this->get_status_id(true);

		if ($count_only)
			$this->db->select('count(task_id) as over_due_count');
		else
			$this->db->select('*');

		$this->db->from('project_task');
		$this->db->where(array(
			'entity_id'	=> $this->entity_id,
			'task_status_id !='	=> $closed_status_id,
			'assignee_id'		=> $this->user_id,
			'finish_date <'		=> date('Y-m-d')
		));

		if ($count_only)
		{
			$result	= $this->db->get()->row_array();
			$result	= !empty($result) && !empty($result['over_due_count']) ? $result['over_due_count'] : 0;
		}
		else
		{
			$this->db->order_by('task_priority, finish_date');
			$result	= $this->db->get()->result_array();
		}

		return $result;
	}

	public function get_due_today($count_only=true)
	{
		$closed_status_id	= $this->get_status_id(true);

		if ($count_only)
			$this->db->select('count(task_id) as due_today_count');
		else
			$this->db->select('*');

		$this->db->from('project_task');
		$this->db->where(array(
			'entity_id'	=> $this->entity_id,
			'task_status_id !='	=> $closed_status_id,
			'assignee_id'		=> $this->user_id,
			'finish_date'		=> date('Y-m-d')
		));

		if ($count_only)
		{
			$result	= $this->db->get()->row_array();
			$result	= !empty($result) && !empty($result['due_today_count']) ? $result['due_today_count'] : 0;
		}
		else
		{
			$this->db->order_by('task_priority, finish_date');
			$result	= $this->db->get()->result_array();
		}

		return $result;
	}

	public function get_due_this_week($count_only=true)
	{
		$closed_status_id	= $this->get_status_id(true);

		$week_end	= array('SU'=>'Sunday', 'MO'=>'Monday', 'TU'=>'Tuesday', 'WE'=>'Wednesday', 'TH'=>'Thursday', 'FR'=>'Friday', 'SA'=>'Saturday');
		$week_end	= date('Y-m-d', strtotime('next ' . $week_end[$this->ot_app->get('start_week_on')]));

		if ($count_only)
			$this->db->select('count(task_id) as due_this_week_count');
		else
			$this->db->select('*');

		$this->db->from('project_task');
		$this->db->where(array(
			'entity_id'	=> $this->entity_id,
			'task_status_id !='	=> $closed_status_id,
			'assignee_id'	=> $this->user_id,
			'finish_date <'	=> $week_end,
			'finish_date >'	=> date('Y-m-d')
		));

		if ($count_only)
		{
			$result	= $this->db->get()->row_array();
			$result	= !empty($result) && !empty($result['due_this_week_count']) ? $result['due_this_week_count'] : 0;
		}
		else
		{
			$this->db->order_by('task_priority, finish_date');
			$result	= $this->db->get()->result_array();
		}

		return $result;
	}

	public function get_status_count()
	{
		$this->db->select('pt.task_status_id, count(pt.task_id) as count, ts.name, ts.text_color, ts.bg_color, ts.is_closed, ts.system_defined');
		$this->db->from('project_task as pt');
		$this->db->join('task_status as ts', 'pt.task_status_id=ts.task_status_id');
		$this->db->where(array(
			'pt.entity_id'	=> $this->entity_id,
			'pt.assignee_id'	=> $this->user_id,
		));
		$this->db->group_by('pt.task_status_id');
		$this->db->order_by('count', 'desc');
		$result	= $this->db->get()->result_array();

		return $result;
	}

	/***
	 * $filter = filter id for status.
	 * 4 = Open
	 * 5 = Active
	 */
	public function get_tasks_by_status ($select= '', $status_id=0, $is_closed = 1)
	{
		/*if (!$status_id)
			return false;*/

		$select = ($select)?$select:'*';

		$where = array(
			'entity_id'	=> $this->entity_id,
			'assignee_id'	=> $this->user_id,
			'is_closed !='	=> $is_closed
		);

		if ($status_id) {
			$where['task_status_id']	= $status_id;
		}

		$this->db->select($select);
		$this->db->from('project_task');
		$this->db->where($where);
		$this->db->order_by('task_priority, finish_date');
		$result	= $this->db->get()->result_array();

		return $result;
	}

	public function get_task_status()
	{
		$this->db->select('*')->from('task_status');
		$this->db->where(array(
			'entity_id'		=> $this->entity_id
		));
		$result	= $this->db->get()->result_array();

		return $result;
	}

	public function get_all_task_types()
	{
		return $this->db->select('task_type_id, name')->from('task_type')->where(array(
			'entity_id'	=> $this->entity_id
		))->get()->result_array();
	}

	public function get_time_sheet($task_id=0)
	{
		$where	= array(
			'entity_id'	=> $this->entity_id,
			'user_id'	=> $this->user_id,
			'is_progress'	=> '1'
		);

		if (!empty($task_id))
		{
			$where['project_task_id']	= $task_id;
		}
		else
		{
			$where['project_task_id != ']	= 0;
		}

		$return	= $this->db->select('project_task_id, time_sheet_id, hours, entry_date')->from('time_sheet')->where($where)->get()->result_array();
		if (!empty($return) && !empty($return[0]))
		{
			return $return[0];
		}

		return false;
	}

	public function get_time_sheet_detail($time_sheet_id=0)
	{
		$where	= array(
			'entity_id'	=> $this->entity_id,
			'time_sheet_id'	=> $time_sheet_id,
			'is_progress'	=> '1'
		);

		$return	= $this->db->select('time_sheet_detail_id, from_time')->from('time_sheet_detail')->where($where)->get()->result_array();

		if (!empty($return) && !empty($return[0]))
		{
			$return	= $return[0];
			$return['from_time'] = time_in_12_hour_format($return['from_time'], true);

			return $return;
		}

		return false;
	}
}
